%%%%% FUNCTION CHBEVALV.M %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                          %
%  S R Lau                                                 %
%  DAM Brown University                                    %
%  Providence, RI 02906                                    %
%                                                          %
%  Code last modified: 18 November 2006.                   %
%                                                          %
%  Just a rewrite of the chbeval code for which u is taken %
%  as a column rather than row vectors, whence the "v" for %
%  vertical.                                               %
%                                                          %
%  For an array u of function values on a Chebyshev grid,  %
%  function returns interpolated values on another grid    %
%  specified as input. See chbdiff.m for other comments.   %
%  x = 0.5(b-a)z + 0.5(a+b), where x is array of Chebyshev %
%  nodes, defined below, so x(-1) = a and x(1) = b.        %
%                                                          %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
      function [u0]=chbevalv(u,x0,a,b)
      u0 = zeros(size(x0));
      n0 = length(x0);
      n = length(u);
      nm1 = n-1;
%
%  Compute the Chebyshev nodes on [-1,1]. Here we include
%  the endpoints x = -1,1. Can comment out if passed.
%
      z = zeros(size(u));
      z = cos(pi*(nm1:-1:0)/nm1);  
%
%  Error message if array is too small.
%
      if nm1 == 0
         error = 'Size of array u only 1 in chbeval.'
         pause
      end
%
%  Array u(x_i) for 1,n corresponds to the left right flip
%  of an array U(phi_i) where i=1,n and 
%
%  phi_i = 2pi(i-1)/(2n-2).
%
%  We want to extend this array to i = 1,2n-2, so that phi_i 
%  corresponds to the usual equally spaced associated with 
%  the FFT. Extend the array so it is even about phi = pi
%  (the extended array will then have a cosine expansion).
%  Here is the extended array. The fliplr is because, x
%  over [-1,1] corresponds to phi over [pi,0] (flipped). We
%  then add u(2:nm1) only, since u(n) would be be U(0) again
%  (for FFT we always include value at 0 not 2pi). After we
%  get U we take the FFT.
%
      U    = vertcat(flipud(u),u(2:nm1));
      Uhat = real(fft(U));
      for p = 1:n0
          z0 = 2*(x0(p)-0.5*(a+b))/(b-a);
          if abs(z0) > 1
             z0 = z0/abs(z0);
          end
          phi0 = acos(z0);
          sum = real(Uhat(1))/(2*n-2);
          for k = 2:n-1
              sum = sum ...
                  + real(Uhat(k))*cos((k-1)*phi0)/(n-1);
          end
          u0(p) = sum;
      end
